"""
Required Notice: Copyright (C) Zoomer Analytics GmbH.

xlwings PRO is dual-licensed under one of the following licenses:

* PolyForm Noncommercial License 1.0.0 (for noncommercial use):
  https://polyformproject.org/licenses/noncommercial/1.0.0
* xlwings PRO License (for commercial use):
  https://github.com/xlwings/xlwings/blob/main/LICENSE_PRO.txt

Commercial licenses can be purchased at https://www.xlwings.org
"""

import logging
from ... import XlwingsError

try:
    import pdfrw
except ImportError:
    pdfrw = None


def print_on_layout(report_path, layout_path):
    if not pdfrw:
        raise XlwingsError(
            "Couldn't find the 'pdfrw' package, which is required when using 'layout'."
        )
    report = pdfrw.PdfReader(report_path)
    layout = pdfrw.PdfReader(layout_path)

    for ix, page in enumerate(report.pages):
        if len(layout.pages) == 1:
            # Same layout for whole report
            layout_page_ix = 0
        elif len(report.pages) == len(layout.pages):
            # Every report page has a corresponding page in the layout
            layout_page_ix = ix
        else:
            raise XlwingsError(
                "The layout PDF must either be a single page or have the "
                f"same number of pages as the report (report: {len(report.pages)}, "
                f"layout: {len(layout.pages)})"
            )
        merge = pdfrw.PageMerge().add(layout.pages[layout_page_ix])[0]
        pdfrw.PageMerge(page).add(merge, prepend=True).render()
    # Changing log level as the exported PDFs from Excel aren't fully compliant and
    # would log the following:
    # [WARNING] tokens.py:221 Did not find PDF object (12, 0) (...)
    logging.getLogger("pdfrw").setLevel(logging.CRITICAL)
    pdfrw.PdfWriter(report_path, trailer=report).write()



from fastapi import Body
import xlwings as xw

from app import app


@app.post("/hello")
def hello(data: dict = Body(...)):
    # Instantiate a Book object with the deserialized request body
    book = xw.Book(json=data)

    # Use xlwings as usual
    sheet = book.sheets[0]
    if sheet["A1"].value == "Hello xlwings!":
        sheet["A1"].value = "Bye xlwings!"
    else:
        sheet["A1"].value = "Hello xlwings!"

    # Pass the following back as the response
    return book.json()


if __name__ == "__main__":
    import uvicorn

    uvicorn.run("main:app", host="127.0.0.1", port=8000, reload=True)



try:
    import pandas as pd
except ImportError:
    pd = None
try:
    import numpy as np
except ImportError:
    np = None

from .framework import (
    ConversionContext,
    Options,
    Pipeline,
    Converter,
    accessors,
    Accessor,
)

from .standard import (
    DictConverter,
    Accessor,
    RangeAccessor,
    RawValueAccessor,
    ValueAccessor,
    AdjustDimensionsStage,
    CleanDataForWriteStage,
    CleanDataFromReadStage,
    Ensure2DStage,
    ExpandRangeStage,
    ReadValueFromRangeStage,
    TransposeStage,
    WriteValueToRangeStage,
    Options,
    Pipeline,
)

if np:
    from .numpy_conv import NumpyArrayConverter
if pd:
    from .pandas_conv import PandasDataFrameConverter, PandasSeriesConverter
from .. import LicenseError

try:
    from ..pro.reports.markdown import MarkdownConverter, Markdown

    MarkdownConverter.register(Markdown)
except (ImportError, LicenseError):
    pass


def read(rng, value, options):
    convert = options.get("convert", None)
    pipeline = accessors.get(convert, convert).reader(options)
    ctx = ConversionContext(rng=rng, value=value)
    pipeline(ctx)
    return ctx.value


def write(value, rng, options):
    # Don't allow to write lists and tuples as jagged arrays as appscript and pywin32
    # don't handle that properly. This should really be handled in Ensure2DStage, but
    # we'd have to set the original format in the conversion ctx meta as the check
    # should only run for list and tuples.
    if (
        isinstance(value, (list, tuple))
        and len(value) > 0
        and isinstance(value[0], (list, tuple))
    ):
        first_row = value[0]
        for row in value:
            if len(first_row) != len(row):
                raise Exception(
                    "All elements of a 2d list or tuple must be of the same length"
                )
    convert = options.get("convert", None)
    pipeline = (
        accessors.get(convert, convert).router(value, rng, options).writer(options)
    )
    ctx = ConversionContext(rng=rng, value=value)
    pipeline(ctx)
    return ctx.value
